using System;
using System.Reflection;
using HarmonyLib;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;
using Vintagestory.GameContent;

namespace WaypointColours {
	public class WaypointColoursSystem: ModSystem {
		public const string configFilename = "waypointcolours.json";
		protected const string harmonyId = "waypointcolours";
		
		public static WaypointColoursConfig config;
		public static int[] colours = null;

		protected Harmony harmony;

		public override void StartClientSide(ICoreClientAPI capi) {
			base.StartClientSide(capi);

			// Load or create config file.
			try {
				WaypointColoursSystem.config = capi.LoadModConfig<WaypointColoursConfig>(WaypointColoursSystem.configFilename);
			} catch(Exception) {}
			if(WaypointColoursSystem.config == null) {
				// Create new config file.
				WaypointColoursSystem.config = new WaypointColoursConfig();
				capi.StoreModConfig<WaypointColoursConfig>(WaypointColoursSystem.config, WaypointColoursSystem.configFilename);
			}

			WaypointColoursSystem.colours = new int[WaypointColoursSystem.config.colours.Length];
			for(int i = 0; i < WaypointColoursSystem.colours.Length; i ++) {
				WaypointColoursSystem.colours[i] = ColorUtil.Hex2Int("#" + WaypointColoursSystem.config.colours[i]);
			}

			if(WaypointColoursSystem.colours != null) {
				this.harmony = new Harmony(harmonyId);
				harmony.PatchAll(Assembly.GetExecutingAssembly());
			}
		}

		public override void Dispose() {
			base.Dispose();

			if(harmony != null) {
				this.harmony.UnpatchAll(harmonyId);
			}
		}
	}

	public class WaypointColoursConfig {
		public String[] colours = {
			"FFFFFF",
			"FFBFBF",
			"FFD2BF",
			"FFE5BF",
			"FFF8BF",
			"F2FFBF",
			"DFFFBF",
			"CCFFBF",
			"BFFFC5",
			"BFFFD8",
			"BFFFEB",
			"BFBFBF",
			"BFFFFF",
			"BFEBFF",
			"BFD8FF",
			"BFC5FF",
			"CCBFFF",
			"DFBFFF",
			"F2BFFF",
			"FFBFF8",
			"FFBFE5",
			"FFBFD2",
			"7F7F7F",
			"FF3F3F",
			"FF793F",
			"FFB23F",
			"FFEB3F",
			"D8FF3F",
			"9FFF3F",
			"65FF3F",
			"3FFF52",
			"3FFF8C",
			"3FFFC5",
			"3F3F3F",
			"3FFFFF",
			"3FC5FF",
			"3F8CFF",
			"3F52FF",
			"663FFF",
			"9F3FFF",
			"D83FFF",
			"FF3FEB",
			"FF3FB2",
			"FF3F79"
		};
	}

	[HarmonyPatch(typeof(WaypointMapLayer), MethodType.Constructor, new Type[] { typeof(ICoreAPI), typeof(IWorldMapManager) })]
	public class WC_WML_Constructor {
		static void Postfix(WaypointMapLayer __instance) {
			__instance.WaypointColors.Clear();

			for(int i = 0; i < WaypointColoursSystem.colours.Length; i ++) {
				__instance.WaypointColors.Add(WaypointColoursSystem.colours[i]);
			}
		}
	}
}